#include <stdio.h>
#include <stdlib.h>

#ifdef DICT_HW_MEASURE_TIMINGS
# include <sys/time.h>
#endif /* DICT_HW_MEASURE_TIMINGS */

#include "normxcorr_hw.h"
#include "normxcorr_hw_msg.h"

#include "helpers.h"

#include "dict_image.h"

int dictImageFreeBuffer(DICTContext ctx, DICTImageType image_type, void *image_buf) {
    switch (image_type) {
#ifdef DICT_TIFF_SUPPORT
        case DICT_IMAGE_TIFF:
	    _TIFFfree(image_buf);
	break;
#endif /* DICT_TIFF_SUPPORT */
    }
}

int dictImageFree(DICTContext ps) {
    if (ps->image_buf) dictImageFreeBuffer(ps, ps->image_type, ps->image_buf);
    if (ps->image) free(ps->image);
}

int dictReduceImage(DICTContext ps, void *img, int width, int height) {
    int size = width * height;
    unsigned char *res = ps->image;

#ifdef DICT_TIFF_SUPPORT
    unsigned char *image = (unsigned char*)img;
#endif /* DICT_TIFF_SUPPORT */

    switch (ps->image_type) {
#ifdef DICT_TIFF_SUPPORT
        case DICT_IMAGE_TIFF:
	    if (ps->matlab_mode) {
		for (int i = 0; i < size; ++i) {
		    int col = i % height;
		    int lin = i / height;
		    int pos = 4*(col * width + lin);
		    res[i] = rintf(((double)(image[pos] + image[pos+1] + image[pos+2]))/3);
		}
	    } else {
	        for (int i = 0; i < size; ++i) {
		    int pos = 4 * i;
		    res[i] = rintf(((double)(image[pos] + image[pos+1] + image[pos+2]))/3);
		}
	    }
	break;
#endif /* DICT_TIFF_SUPPORT */
    }
}

int dictImageLoadTemplateImage(DICTContext ps, const char *name) {
#ifdef DICT_TIFF_SUPPORT
    uint32_t w, h;

#ifdef DICT_HW_MEASURE_TIMINGS
    int time;
    struct timeval tv1, tv2, tv3;
    gettimeofday(&tv1, NULL);
#endif /* DICT_HW_MEASURE_TIMINGS */
    
    TIFF* tif = TIFFOpen(name, "r");
    if (!tif) return DICT_ERROR_IMAGE;
    
    TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &w);
    TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
    
    if (ps->image_buf) {
	dictImageFreeBuffer(ps, ps->image_type, ps->image_buf);
    }
    
    if (ps->image) {
	free(ps->image);
    }

    ps->image_type = DICT_IMAGE_TIFF;
    ps->image_buf = _TIFFmalloc(w * h * sizeof (uint32_t));
    ps->image = (unsigned char*)malloc(w * h * sizeof(unsigned char));
    
    ps->width = w;
    ps->height = h;
    
    if ((!ps->image)||(!ps->image_buf)) {
	reportError("Problem allocating memory for image %ix%i", w, h);
	return DICT_ERROR_MALLOC;
    }

    if (TIFFReadRGBAImage(tif, w, h, (uint32_t*)ps->image_buf, 0)) {
#ifdef DICT_HW_MEASURE_TIMINGS
	gettimeofday(&tv3, NULL);
#endif /* DICT_HW_MEASURE_TIMINGS */
	dictReduceImage(ps, ps->image_buf, w, h);
#ifdef DICT_HW_MEASURE_TIMINGS
	gettimeofday(&tv2, NULL);
	time = (tv2.tv_sec - tv3.tv_sec) * 1000000 + (tv2.tv_usec - tv3.tv_usec);
	ps->time[13] += time;
#endif /* DICT_HW_MEASURE_TIMINGS */

    }
    
    TIFFClose(tif);

#ifdef DICT_HW_MEASURE_TIMINGS
    gettimeofday(&tv2, NULL);
    ps->time[12] += (tv2.tv_sec - tv1.tv_sec) * 1000000 + (tv2.tv_usec - tv1.tv_usec) - time;
#endif /* DICT_HW_MEASURE_TIMINGS */

    return 0;
#else
    return DICT_ERROR_IMAGE;
#endif /* DICT_TIFF_SUPPORT */
}

int dictImageLoadImage(DICTContext ps, const char *name) {
#ifdef DICT_TIFF_SUPPORT
#ifdef DICT_HW_MEASURE_TIMINGS
    int time;
    struct timeval tv1, tv2, tv3;
    gettimeofday(&tv1, NULL);
#endif /* DICT_HW_MEASURE_TIMINGS */

    TIFF* tif = TIFFOpen(name, "r");
    if (!tif) return DICT_ERROR_IMAGE;

    if (TIFFReadRGBAImage(tif, ps->width, ps->height, (uint32_t*)ps->image_buf, 0)) {
#ifdef DICT_HW_MEASURE_TIMINGS
	gettimeofday(&tv3, NULL);
#endif /* DICT_HW_MEASURE_TIMINGS */
	dictReduceImage(ps, ps->image_buf, ps->width, ps->height);
#ifdef DICT_HW_MEASURE_TIMINGS
	gettimeofday(&tv2, NULL);
	time = (tv2.tv_sec - tv3.tv_sec) * 1000000 + (tv2.tv_usec - tv3.tv_usec);
	ps->time[15] += time;
#endif /* DICT_HW_MEASURE_TIMINGS */
    }

    TIFFClose(tif);

#ifdef DICT_HW_MEASURE_TIMINGS
    gettimeofday(&tv2, NULL);
    ps->time[14] += (tv2.tv_sec - tv1.tv_sec) * 1000000 + (tv2.tv_usec - tv1.tv_usec) - time;
#endif /* DICT_HW_MEASURE_TIMINGS */
    return 0;
#else
    return DICT_ERROR_IMAGE;
#endif /* DICT_TIFF_SUPPORT */
}
